@@ -1,8 +1,6 @@
 //
 // Binder.h
 //
-// $Id: //poco/Main/Data/ODBC/include/Poco/Data/ODBC/Binder.h#5 $
-//
 // Library: Data/ODBC
 // Package: ODBC
 // Module:  Binder
@@ -73,7 +71,9 @@ public:
 	Binder(const StatementHandle& rStmt,
 		std::size_t maxFieldSize,
 		ParameterBinding dataBinding = PB_IMMEDIATE,
-		TypeInfo* pDataTypes = 0);
+		const TypeInfo* pDataTypes = 0,
+		Poco::TextEncoding::Ptr pFromEncoding = nullptr,
+		Poco::TextEncoding::Ptr pDBEncoding = nullptr);
 		/// Creates the Binder.
 
 	~Binder();
@@ -105,7 +105,7 @@ public:
 
 	void bind(std::size_t pos, const Poco::Int16& val, Direction dir);
 		/// Binds an Int16.
-	
+
 	void bind(std::size_t pos, const std::vector<Poco::Int16>& val, Direction dir);
 		/// Binds an Int16 vector.
 
@@ -175,7 +175,7 @@ public:
 	void bind(std::size_t pos, const std::list<Poco::UInt64>& val, Direction dir);
 		/// Binds an UInt64 list.
 
-#ifndef POCO_LONG_IS_64_BIT
+#ifndef POCO_INT64_IS_LONG
 	void bind(std::size_t pos, const long& val, Direction dir);
 		/// Binds a long.
 
@@ -324,6 +324,9 @@ public:
 	void bind(std::size_t pos, const std::list<DateTime>& val, Direction dir);
 		/// Binds a DateTime list.
 
+	void bind(std::size_t pos, const UUID& val, Direction dir);
+		/// Binds a UUID.
+
 	void bind(std::size_t pos, const NullData& val, Direction dir);
 		/// Binds a null. In-bound only.
 
@@ -346,7 +349,7 @@ public:
 		/// Returns bound data size for parameter at specified position.
 
 	void synchronize();
-		/// Transfers the results of non-POD outbound parameters from internal 
+		/// Transfers the results of non-POD outbound parameters from internal
 		/// holders back into the externally supplied buffers.
 
 	void reset();
@@ -369,6 +372,7 @@ private:
 	typedef std::vector<AnyVec>                              AnyVecVec;
 	typedef std::map<char*, std::string*>                    StringMap;
 	typedef std::map<UTF16String::value_type*, UTF16String*> UTF16StringMap;
+	typedef std::map<char*, UUID*>                           UUIDMap;
 	typedef std::map<SQL_DATE_STRUCT*, Date*>                DateMap;
 	typedef std::map<SQL_TIME_STRUCT*, Time*>                TimeMap;
 	typedef std::map<SQL_TIMESTAMP_STRUCT*, DateTime*>       TimestampMap;
@@ -377,7 +381,7 @@ private:
 		/// Sets the description field for the parameter, if needed.
 
 	void bind(std::size_t pos, const char* const& pVal, Direction dir);
-		/// Binds a const char ptr. 
+		/// Binds a const char ptr.
 		/// This is a private no-op in this implementation
 		/// due to security risk.
 
@@ -394,11 +398,11 @@ private:
 
 		_lengthIndicator.push_back(0);
 
-		if (Utility::isError(SQLBindParameter(_rStmt, 
-			(SQLUSMALLINT) pos + 1, 
-			toODBCDirection(dir), 
-			cDataType, 
-			Utility::sqlDataType(cDataType), 
+		if (Utility::isError(SQLBindParameter(_rStmt,
+			(SQLUSMALLINT) pos + 1,
+			toODBCDirection(dir),
+			cDataType,
+			Utility::sqlDataType(cDataType),
 			colSize,
 			decDigits,
 			(SQLPOINTER) &val, 0, 0)))
@@ -415,26 +419,28 @@ private:
 
 		SQLPOINTER pVal = (SQLPOINTER) val.rawContent();
 		SQLINTEGER size = (SQLINTEGER) val.size();
-			
+
 		_inParams.insert(ParamMap::value_type(pVal, size));
 
 		SQLLEN* pLenIn = new SQLLEN;
 		*pLenIn  = size;
+		// prevent SQLSTATE = HY104 "Invalid precision value"
+		SQLULEN columnSize = size ? size : 1;
 
 		if (PB_AT_EXEC == _paramBinding)
 			*pLenIn  = SQL_LEN_DATA_AT_EXEC(size);
 
 		_lengthIndicator.push_back(pLenIn);
 
-		if (Utility::isError(SQLBindParameter(_rStmt, 
-			(SQLUSMALLINT) pos + 1, 
-			SQL_PARAM_INPUT, 
-			SQL_C_BINARY, 
-			SQL_LONGVARBINARY, 
-			(SQLUINTEGER) size,
+		if (Utility::isError(SQLBindParameter(_rStmt,
+			(SQLUSMALLINT) pos + 1,
+			SQL_PARAM_INPUT,
+			SQL_C_BINARY,
+			SQL_LONGVARBINARY,
+			columnSize,
 			0,
 			pVal,
-			(SQLINTEGER) size, 
+			(SQLINTEGER) size,
 			_lengthIndicator.back())))
 		{
 			throw StatementException(_rStmt, "SQLBindParameter(LOB)");
@@ -459,15 +465,15 @@ private:
 			_vecLengthIndicator[pos] = new LengthVec(length);
 		}
 
-		if (Utility::isError(SQLBindParameter(_rStmt, 
-			(SQLUSMALLINT) pos + 1, 
-			toODBCDirection(dir), 
-			cDataType, 
-			Utility::sqlDataType(cDataType), 
+		if (Utility::isError(SQLBindParameter(_rStmt,
+			(SQLUSMALLINT) pos + 1,
+			toODBCDirection(dir),
+			cDataType,
+			Utility::sqlDataType(cDataType),
 			colSize,
 			decDigits,
-			(SQLPOINTER) &val[0], 
-			0, 
+			(SQLPOINTER) &val[0],
+			0,
 			&(*_vecLengthIndicator[pos])[0])))
 		{
 			throw StatementException(_rStmt, "SQLBindParameter()");
@@ -520,15 +526,15 @@ private:
 		typename C::const_iterator end = val.end();
 		for (int i = 0; it != end; ++it, ++i) _boolPtrs[pos][i] = *it;
 
-		if (Utility::isError(SQLBindParameter(_rStmt, 
-			(SQLUSMALLINT) pos + 1, 
-			toODBCDirection(dir), 
-			cDataType, 
-			Utility::sqlDataType(cDataType), 
+		if (Utility::isError(SQLBindParameter(_rStmt,
+			(SQLUSMALLINT) pos + 1,
+			toODBCDirection(dir),
+			cDataType,
+			Utility::sqlDataType(cDataType),
 			colSize,
 			decDigits,
-			(SQLPOINTER) &_boolPtrs[pos][0], 
-			0, 
+			(SQLPOINTER) &_boolPtrs[pos][0],
+			0,
 			&(*_vecLengthIndicator[pos])[0])))
 		{
 			throw StatementException(_rStmt, "SQLBindParameter()");
@@ -536,7 +542,7 @@ private:
 	}
 
 	template <typename C>
-	void bindImplContainerString(std::size_t pos, const C& val, Direction dir)
+	void bindImplContainerString(std::size_t pos, const C& valC, Direction dir)
 		/// Utility function to bind containers of strings.
 	{
 		if (isOutBound(dir) || !isInBound(dir))
@@ -545,7 +551,19 @@ private:
 		if (PB_IMMEDIATE != _paramBinding)
 			throw InvalidAccessException("Containers can only be bound immediately.");
 
-		std::size_t length = val.size();
+		const C* pVal = 0;
+		if (!transcodeRequired()) pVal = &valC;
+		else
+		{
+			pVal = new C(valC.size());
+			typename C::const_iterator valIt = valC.begin();
+			typename C::const_iterator valEnd = valC.end();
+			typename C::iterator tcIt = const_cast<C*>(pVal)->begin();
+			for (; valIt != valEnd; ++valIt, ++tcIt)
+				transcode(*valIt, *tcIt);
+		}
+
+		std::size_t length = pVal->size();
 
 		if (0 == length)
 			throw InvalidArgumentException("Empty container not allowed.");
@@ -558,7 +576,7 @@ private:
 
 		if (size == _maxFieldSize)
 		{
-			getMinValueSize(val, size);
+			getMinValueSize(*pVal, size);
 			// accomodate for terminating zero
 			if (size != _maxFieldSize) ++size;
 		}
@@ -572,33 +590,38 @@ private:
 		if (_charPtrs.size() <= pos)
 			_charPtrs.resize(pos + 1, 0);
 
-		_charPtrs[pos] = (char*) std::calloc(val.size() * size, sizeof(char));
-		
+		_charPtrs[pos] = (char*) std::calloc(pVal->size() * size, sizeof(char));
+
+		std::string typeID = typeid(*pVal).name();
 		std::size_t strSize;
 		std::size_t offset = 0;
-		typename C::const_iterator it = val.begin();
-		typename C::const_iterator end = val.end();
+		typename C::const_iterator it = pVal->begin();
+		typename C::const_iterator end = pVal->end();
 		for (; it != end; ++it)
 		{
 			strSize = it->size();
-			if (strSize > size)	
-				throw LengthExceededException("SQLBindParameter(std::vector<std::string>)");
+			if (strSize > size)
+			{
+				if (transcodeRequired()) delete pVal;
+				throw LengthExceededException(Poco::format("SQLBindParameter(%s)", typeID));
+			}
 			std::memcpy(_charPtrs[pos] + offset, it->c_str(), strSize);
 			offset += size;
 		}
+		if (transcodeRequired()) delete pVal;
 
-		if (Utility::isError(SQLBindParameter(_rStmt, 
-			(SQLUSMALLINT) pos + 1, 
-			toODBCDirection(dir), 
-			SQL_C_CHAR, 
-			SQL_LONGVARCHAR, 
+		if (Utility::isError(SQLBindParameter(_rStmt,
+			(SQLUSMALLINT) pos + 1,
+			toODBCDirection(dir),
+			SQL_C_CHAR,
+			SQL_LONGVARCHAR,
 			(SQLUINTEGER) size - 1,
 			0,
-			_charPtrs[pos], 
-			(SQLINTEGER) size, 
+			_charPtrs[pos],
+			(SQLINTEGER) size,
 			&(*_vecLengthIndicator[pos])[0])))
 		{
-			throw StatementException(_rStmt, "SQLBindParameter(std::vector<std::string>)");
+			throw StatementException(_rStmt, Poco::format("SQLBindParameter(%s)", typeID));
 		}
 	}
 
@@ -642,6 +665,7 @@ private:
 
 		std::size_t strSize;
 		std::size_t offset = 0;
+		char* pBuf = (char*)_utf16CharPtrs[pos];
 		typename C::const_iterator it = val.begin();
 		typename C::const_iterator end = val.end();
 		for (; it != end; ++it)
@@ -649,8 +673,8 @@ private:
 			strSize = it->size() * sizeof(UTF16Char);
 			if (strSize > size)
 				throw LengthExceededException("SQLBindParameter(std::vector<UTF16String>)");
-			std::memcpy(_utf16CharPtrs[pos] + offset, it->data(), strSize);
-			offset += (size / sizeof(UTF16Char));
+			std::memcpy(pBuf + offset, it->data(), strSize);
+			offset += size;
 		}
 
 		if (Utility::isError(SQLBindParameter(_rStmt,
@@ -697,7 +721,7 @@ private:
 		std::vector<SQLLEN>::iterator lIt = _vecLengthIndicator[pos]->begin();
 		std::vector<SQLLEN>::iterator lEnd = _vecLengthIndicator[pos]->end();
 		typename C::const_iterator cIt = val.begin();
-		for (; lIt != lEnd; ++lIt, ++cIt) 
+		for (; lIt != lEnd; ++lIt, ++cIt)
 		{
 			SQLLEN sz = static_cast<SQLLEN>(cIt->size());
 			if (sz > size) size = static_cast<SQLINTEGER>(sz);
@@ -717,21 +741,21 @@ private:
 		for (; cIt != cEnd; ++cIt)
 		{
 			blobSize = cIt->size();
-			if (blobSize > size)	
+			if (blobSize > size)
 				throw LengthExceededException("SQLBindParameter(std::vector<BLOB>)");
 			std::memcpy(_charPtrs[pos] + offset, cIt->rawContent(), blobSize * sizeof(CharType));
 			offset += size;
 		}
 
-		if (Utility::isError(SQLBindParameter(_rStmt, 
-			(SQLUSMALLINT) pos + 1, 
-			SQL_PARAM_INPUT, 
-			SQL_C_BINARY, 
-			SQL_LONGVARBINARY, 
+		if (Utility::isError(SQLBindParameter(_rStmt,
+			(SQLUSMALLINT) pos + 1,
+			SQL_PARAM_INPUT,
+			SQL_C_BINARY,
+			SQL_LONGVARBINARY,
 			(SQLUINTEGER) size,
 			0,
-			_charPtrs[pos], 
-			(SQLINTEGER) size, 
+			_charPtrs[pos],
+			(SQLINTEGER) size,
 			&(*_vecLengthIndicator[pos])[0])))
 		{
 			throw StatementException(_rStmt, "SQLBindParameter(std::vector<BLOB>)");
@@ -754,8 +778,6 @@ private:
 
 		setParamSetSize(length);
 
-		SQLINTEGER size = (SQLINTEGER) sizeof(SQL_DATE_STRUCT);
-
 		if (_vecLengthIndicator.size() <= pos)
 		{
 			_vecLengthIndicator.resize(pos + 1, 0);
@@ -774,15 +796,15 @@ private:
 		SQLSMALLINT decDigits = 0;
 		getColSizeAndPrecision(pos, SQL_TYPE_DATE, colSize, decDigits);
 
-		if (Utility::isError(SQLBindParameter(_rStmt, 
-			(SQLUSMALLINT) pos + 1, 
-			toODBCDirection(dir), 
-			SQL_C_TYPE_DATE, 
-			SQL_TYPE_DATE, 
+		if (Utility::isError(SQLBindParameter(_rStmt,
+			(SQLUSMALLINT) pos + 1,
+			toODBCDirection(dir),
+			SQL_C_TYPE_DATE,
+			SQL_TYPE_DATE,
 			colSize,
 			decDigits,
-			(SQLPOINTER) &(*_dateVecVec[pos])[0], 
-			0, 
+			(SQLPOINTER) &(*_dateVecVec[pos])[0],
+			0,
 			&(*_vecLengthIndicator[pos])[0])))
 		{
 			throw StatementException(_rStmt, "SQLBindParameter(Date[])");
@@ -804,8 +826,6 @@ private:
 
 		setParamSetSize(val.size());
 
-		SQLINTEGER size = (SQLINTEGER) sizeof(SQL_TIME_STRUCT);
-
 		if (_vecLengthIndicator.size() <= pos)
 		{
 			_vecLengthIndicator.resize(pos + 1, 0);
@@ -824,15 +844,15 @@ private:
 		SQLSMALLINT decDigits = 0;
 		getColSizeAndPrecision(pos, SQL_TYPE_TIME, colSize, decDigits);
 
-		if (Utility::isError(SQLBindParameter(_rStmt, 
-			(SQLUSMALLINT) pos + 1, 
-			toODBCDirection(dir), 
-			SQL_C_TYPE_TIME, 
-			SQL_TYPE_TIME, 
+		if (Utility::isError(SQLBindParameter(_rStmt,
+			(SQLUSMALLINT) pos + 1,
+			toODBCDirection(dir),
+			SQL_C_TYPE_TIME,
+			SQL_TYPE_TIME,
 			colSize,
 			decDigits,
-			(SQLPOINTER) &(*_timeVecVec[pos])[0], 
-			0, 
+			(SQLPOINTER) &(*_timeVecVec[pos])[0],
+			0,
 			&(*_vecLengthIndicator[pos])[0])))
 		{
 			throw StatementException(_rStmt, "SQLBindParameter(Time[])");
@@ -855,8 +875,6 @@ private:
 
 		setParamSetSize(length);
 
-		SQLINTEGER size = (SQLINTEGER) sizeof(SQL_TIMESTAMP_STRUCT);
-
 		if (_vecLengthIndicator.size() <= pos)
 		{
 			_vecLengthIndicator.resize(pos + 1, 0);
@@ -875,15 +893,15 @@ private:
 		SQLSMALLINT decDigits = 0;
 		getColSizeAndPrecision(pos, SQL_TYPE_TIMESTAMP, colSize, decDigits);
 
-		if (Utility::isError(SQLBindParameter(_rStmt, 
-			(SQLUSMALLINT) pos + 1, 
-			toODBCDirection(dir), 
-			SQL_C_TYPE_TIMESTAMP, 
-			SQL_TYPE_TIMESTAMP, 
+		if (Utility::isError(SQLBindParameter(_rStmt,
+			(SQLUSMALLINT) pos + 1,
+			toODBCDirection(dir),
+			SQL_C_TYPE_TIMESTAMP,
+			SQL_TYPE_TIMESTAMP,
 			colSize,
 			decDigits,
-			(SQLPOINTER) &(*_dateTimeVecVec[pos])[0], 
-			0, 
+			(SQLPOINTER) &(*_dateTimeVecVec[pos])[0],
+			0,
 			&(*_vecLengthIndicator[pos])[0])))
 		{
 			throw StatementException(_rStmt, "SQLBindParameter(Time[])");
@@ -906,8 +924,6 @@ private:
 
 		setParamSetSize(length);
 
-		SQLINTEGER size = SQL_NULL_DATA;
-
 		if (_vecLengthIndicator.size() <= pos)
 		{
 			_vecLengthIndicator.resize(pos + 1, 0);
@@ -918,32 +934,35 @@ private:
 		SQLSMALLINT decDigits = 0;
 		getColSizeAndPrecision(pos, SQL_C_STINYINT, colSize, decDigits);
 
-		if (Utility::isError(SQLBindParameter(_rStmt, 
-			(SQLUSMALLINT) pos + 1, 
-			SQL_PARAM_INPUT, 
-			SQL_C_STINYINT, 
-			Utility::sqlDataType(SQL_C_STINYINT), 
+		if (Utility::isError(SQLBindParameter(_rStmt,
+			(SQLUSMALLINT) pos + 1,
+			SQL_PARAM_INPUT,
+			SQL_C_STINYINT,
+			Utility::sqlDataType(SQL_C_STINYINT),
 			colSize,
 			decDigits,
-			0, 
-			0, 
+			0,
+			0,
 			&(*_vecLengthIndicator[pos])[0])))
 		{
 			throw StatementException(_rStmt, "SQLBindParameter()");
 		}
 	}
 
-	void getColSizeAndPrecision(std::size_t pos, 
-		SQLSMALLINT cDataType, 
-		SQLINTEGER& colSize, 
-		SQLSMALLINT& decDigits);
+	void getColSizeAndPrecision(std::size_t pos,
+		SQLSMALLINT cDataType,
+		SQLINTEGER& colSize,
+		SQLSMALLINT& decDigits,
+		std::size_t actualSize = 0);
 		/// Used to retrieve column size and precision.
 		/// Not all drivers cooperate with this inquiry under all circumstances
-		/// This function runs for query and stored procedure parameters (in and 
-		/// out-bound). Some drivers, however, do not care about knowing this 
-		/// information to start with. For that reason, after all the attempts 
-		/// to discover the required values are unsuccesfully exhausted, the values 
+		/// This function runs for query and stored procedure parameters (in and
+		/// out-bound). Some drivers, however, do not care about knowing this
+		/// information to start with. For that reason, after all the attempts
+		/// to discover the required values are unsuccesfully exhausted, the values
 		/// are both set to zero and no exception is thrown.
+		/// However, if the colSize is succesfully retrieved and it is greater than
+		/// session-wide maximum allowed field size, LengthExceededException is thrown.
 
 	void setParamSetSize(std::size_t length);
 		/// Sets the parameter set size. Used for column-wise binding.
@@ -951,7 +970,7 @@ private:
 	void getColumnOrParameterSize(std::size_t pos, SQLINTEGER& size);
 		/// Fills the column or parameter size into the 'size' argument.
 		/// Does nothing if neither can be obtained from the driver, so
-		/// size should be set to some default value prior to calling this 
+		/// size should be set to some default value prior to calling this
 		/// function in order to avoid undefined size value.
 
 	void freeMemory();
@@ -960,18 +979,21 @@ private:
 	template<typename T>
 	void getMinValueSize(T& val, SQLINTEGER& size)
 		/// Some ODBC drivers return DB-wide maximum allowed size for variable size columns,
-		/// rather than the allowed size for the actual column. In such cases, the length is 
+		/// rather than the allowed size for the actual column. In such cases, the length is
 		/// automatically resized to the maximum field size allowed by the session.
-		/// This function, in order to prevent unnecessary memory allocation, does further 
+		/// This function, in order to prevent unnecessary memory allocation, does further
 		/// optimization, looking for the maximum length within supplied data container and
 		/// uses the smaller of maximum found and maximum predefined data length.
 	{
+		typedef typename T::value_type ContainedValType;
+		typedef typename ContainedValType::value_type BaseValType;
+		std::size_t typeSize = sizeof(BaseValType);
 		std::size_t maxSize = 0;
 		typename T::const_iterator it = val.begin();
 		typename T::const_iterator end = val.end();
 		for (; it != end; ++it)
 		{
-			std::size_t sz = it->size() * sizeof(T);
+			std::size_t sz = it->size() * typeSize;
 			if (sz > _maxFieldSize)
 				throw LengthExceededException();
 
@@ -995,12 +1017,13 @@ private:
 	ParamMap         _inParams;
 	ParamMap         _outParams;
 	ParameterBinding _paramBinding;
-	
+
 	DateMap          _dates;
 	TimeMap          _times;
 	TimestampMap     _timestamps;
 	StringMap        _strings;
 	UTF16StringMap   _utf16Strings;
+	UUIDMap          _uuids;
 
 	DateVecVec       _dateVecVec;
 	TimeVecVec       _timeVecVec;
@@ -1210,7 +1233,7 @@ inline void Binder::bind(std::size_t pos, const std::list<Poco::UInt64>& val, Di
 }
 
 
-#ifndef POCO_LONG_IS_64_BIT
+#ifndef POCO_INT64_IS_LONG
 inline void Binder::bind(std::size_t pos, const long& val, Direction dir)
 {
 	bindImpl(pos, val, SQL_C_SLONG, dir);
