@@ -21,6 +21,7 @@
 #include "Poco/JSON/JSON.h"
 #include "Poco/JSON/Array.h"
 #include "Poco/JSON/Stringifier.h"
+#include "Poco/JSONString.h"
 #include "Poco/SharedPtr.h"
 #include "Poco/Dynamic/Var.h"
 #include "Poco/Dynamic/Struct.h"
@@ -37,39 +38,45 @@ namespace JSON {
 
 
 class JSON_API Object
-	/// Represents a JSON object. Object provides a representation
-	/// based on shared pointers and optimized for performance. It is possible to 
+	/// Represents a JSON object. Object provides a representation based on
+	/// shared pointers and optimized for performance. It is possible to
 	/// convert Object to DynamicStruct. Conversion requires copying and therefore
 	/// has performance penalty; the benefit is in improved syntax, eg:
-	/// 
+	///
 	///    std::string json = "{ \"test\" : { \"property\" : \"value\" } }";
 	///    Parser parser;
 	///    Var result = parser.parse(json);
-	/// 
+	///
 	///    // use pointers to avoid copying
 	///    Object::Ptr object = result.extract<Object::Ptr>();
 	///    Var test = object->get("test"); // holds { "property" : "value" }
 	///    Object::Ptr subObject = test.extract<Object::Ptr>();
 	///    test = subObject->get("property");
 	///    std::string val = test.toString(); // val holds "value"
-	/// 
+	///
 	///    // copy/convert to Poco::DynamicStruct
 	///    Poco::DynamicStruct ds = *object;
 	///    val = ds["test"]["property"]; // val holds "value"
-	/// ----
+	///
 {
 public:
-	typedef SharedPtr<Object>                   Ptr;
-	typedef std::map<std::string, Dynamic::Var> ValueMap;
-	typedef ValueMap::value_type                ValueType;
-	typedef ValueMap::iterator                  Iterator;
-	typedef ValueMap::const_iterator            ConstIterator;
+	using Ptr = SharedPtr<Object>;
+	using ValueMap = std::map<std::string, Dynamic::Var>;
+	using ValueType = ValueMap::value_type;
+	using Iterator = ValueMap::iterator;
+	using ConstIterator = ValueMap::const_iterator;
+	using NameList = std::vector<std::string>;
 
-	explicit Object(bool preserveInsertionOrder = false);
+	explicit Object(int options = 0);
 		/// Creates an empty Object.
 		///
-		/// If preserveInsertionOrder, object will preserve the items insertion 
-		/// order. Otherwise, items will be sorted by keys.
+		/// If JSON_PRESERVE_KEY_ORDER is specified, the object will
+		/// preserve the items insertion order. Otherwise, items will be
+		/// sorted by keys.
+		///
+		/// If JSON_ESCAPE_UNICODE is specified, when the object is
+		/// stringified, all unicode characters will be escaped in the
+		/// resulting string.
 
 	Object(const Object& copy);
 		/// Creates an Object by copying another one.
@@ -77,41 +84,35 @@ public:
 		/// Struct is not copied to keep the operation as
 		/// efficient as possible (when needed, it will be generated upon request).
 
-#ifdef POCO_ENABLE_CPP11
-
-	Object(Object&& other);
+	Object(Object&& other) noexcept;
 		/// Move constructor
 
-	Object &operator =(Object &&other);
-		// Move asignment operator
-
-#endif // POCO_ENABLE_CPP11
-
-	virtual ~Object();
+	~Object();
 		/// Destroys the Object.
 
-	Object &operator =(const Object &other);
+	Object &operator = (const Object &other);
 		// Assignment operator
 
-	Iterator begin()
-	{
-		return _values.begin();
-	}
+	Object &operator = (Object &&other) noexcept;
+		// Move asignment operator
 
-	ConstIterator begin() const
-	{
-		return _values.begin();
-	}
+	void setEscapeUnicode(bool escape = true);
+		/// Sets the flag for escaping unicode.
 
-	Iterator end()
-	{
-		return _values.end();
-	}
+	bool getEscapeUnicode() const;
+		/// Returns the flag for escaping unicode.
 
-	ConstIterator end() const
-	{
-		return _values.end();
-	}
+	Iterator begin();
+		/// Returns begin iterator for values.
+
+	ConstIterator begin() const;
+		/// Returns const begin iterator for values.
+
+	Iterator end();
+		/// Returns end iterator for values.
+
+	ConstIterator end() const;
+		/// Returns const end iterator for values.
 
 	Dynamic::Var get(const std::string& key) const;
 		/// Retrieves a property. An empty value is
@@ -143,7 +144,7 @@ public:
 	Poco::Nullable<T> getNullableValue(const std::string& key) const
 		/// Retrieves the property with the given name and will
 		/// try to convert the value to the given template type.
-		/// 
+		///
 		/// The convert<T> method of Var is called
 		/// which can also throw exceptions for invalid values.
 		/// Note: This will not work for an array or an object.
@@ -155,7 +156,10 @@ public:
 		return value.convert<T>();
 	}
 
-	void getNames(std::vector<std::string>& names) const;
+	void getNames(NameList& names) const;
+		/// Fills the supplied vector with all property names.
+
+	NameList getNames() const;
 		/// Returns all property names.
 
 	bool has(const std::string& key) const;
@@ -201,13 +205,13 @@ public:
 	std::size_t size() const;
 		/// Returns the number of properties.
 
-	void set(const std::string& key, const Dynamic::Var& value);
+	Object& set(const std::string& key, const Dynamic::Var& value);
 		/// Sets a new value.
 
 	void stringify(std::ostream& out, unsigned int indent = 0, int step = -1) const;
-		/// Prints the object to out stream. 
+		/// Prints the object to out stream.
 		///
-		/// When indent is 0, the object will be printed on a single 
+		/// When indent is 0, the object will be printed on a single
 		/// line without indentation.
 
 	void remove(const std::string& key);
@@ -216,6 +220,12 @@ public:
 	static Poco::DynamicStruct makeStruct(const Object::Ptr& obj);
 		/// Utility function for creation of struct.
 
+	static Poco::OrderedDynamicStruct makeOrderedStruct(const Object::Ptr& obj);
+		/// Utility function for creation of ordered struct.
+
+	operator const Poco::OrderedDynamicStruct& () const;
+		/// Cast operator to Poco::OrderedDynamiStruct.
+
 	operator const Poco::DynamicStruct& () const;
 		/// Cast operator to Poco::DynamiStruct.
 
@@ -225,25 +235,41 @@ public:
 		/// Insertion order preservation property is left intact.
 
 private:
-	void resetDynStruct() const;
+	typedef std::deque<ValueMap::const_iterator>  KeyList;
+	typedef Poco::DynamicStruct::Ptr              StructPtr;
+	typedef Poco::OrderedDynamicStruct::Ptr       OrdStructPtr;
+
+	void syncKeys(const KeyList& keys);
+
+	template <typename T>
+	void resetDynStruct(T& pStruct) const
+	{
+		if (!pStruct)
+			pStruct = new typename T::Type;
+		else
+			pStruct->clear();
+	}
 
 	template <typename C>
 	void doStringify(const C& container, std::ostream& out, unsigned int indent, unsigned int step) const
 	{
+		int options = Poco::JSON_WRAP_STRINGS;
+		options |= _escapeUnicode ? Poco::JSON_ESCAPE_UNICODE : 0;
+
 		out << '{';
 
 		if (indent > 0) out << std::endl;
-		
+
 		typename C::const_iterator it = container.begin();
 		typename C::const_iterator end = container.end();
 		for (; it != end;)
 		{
 			for (unsigned int i = 0; i < indent; i++) out << ' ';
 
-			Stringifier::stringify(getKey(it), out);
-			out << ((indent > 0) ? " : " : ":");
+			Stringifier::stringify(getKey(it), out, indent, step, options);
+			out << ((indent > 0) ? ": " : ":");
 
-			Stringifier::stringify(getValue(it), out, indent + step, step);
+			Stringifier::stringify(getValue(it), out, indent + step, step, options);
 
 			if (++it != container.end()) out << ',';
 
@@ -257,25 +283,118 @@ private:
 		out << '}';
 	}
 
-	typedef std::deque<const std::string*> KeyPtrList;
-	typedef Poco::DynamicStruct::Ptr       StructPtr;
+	template <typename S>
+	static S makeStructImpl(const Object::Ptr& obj)
+	{
+		S ds;
+
+		if (obj->_preserveInsOrder)
+		{
+			KeyList::const_iterator it = obj->_keys.begin();
+			KeyList::const_iterator end = obj->_keys.end();
+			for (; it != end; ++it)
+			{
+				if (obj->isObject((*it)->first))
+				{
+					Object::Ptr pObj = obj->getObject((*it)->first);
+					S str = makeStructImpl<S>(pObj);
+					ds.insert((*it)->first, str);
+				}
+				else if (obj->isArray((*it)->first))
+				{
+					Array::Ptr pArr = obj->getArray((*it)->first);
+					std::vector<Poco::Dynamic::Var> v = Poco::JSON::Array::makeArray(pArr);
+					ds.insert((*it)->first, v);
+				}
+				else
+					ds.insert((*it)->first, (*it)->second);
+			}
+		}
+		else
+		{
+			ConstIterator it = obj->begin();
+			ConstIterator end = obj->end();
+			for (; it != end; ++it)
+			{
+				if (obj->isObject(it))
+				{
+					Object::Ptr pObj = obj->getObject(it->first);
+					S str = makeStructImpl<S>(pObj);
+					ds.insert(it->first, str);
+				}
+				else if (obj->isArray(it))
+				{
+					Array::Ptr pArr = obj->getArray(it->first);
+					std::vector<Poco::Dynamic::Var> v = Poco::JSON::Array::makeArray(pArr);
+					ds.insert(it->first, v);
+				}
+				else
+					ds.insert(it->first, it->second);
+			}
+		}
+
+		return ds;
+	}
 
 	const std::string& getKey(ValueMap::const_iterator& it) const;
 	const Dynamic::Var& getValue(ValueMap::const_iterator& it) const;
-	const std::string& getKey(KeyPtrList::const_iterator& it) const;
-	const Dynamic::Var& getValue(KeyPtrList::const_iterator& it) const;
+	const std::string& getKey(KeyList::const_iterator& it) const;
+	const Dynamic::Var& getValue(KeyList::const_iterator& it) const;
 
 	ValueMap          _values;
-	KeyPtrList        _keys;
+	KeyList           _keys;
 	bool              _preserveInsOrder;
-	mutable StructPtr _pStruct;
-	mutable bool      _modified;
+	// Note:
+	//  The reason for this flag (rather than as argument to stringify()) is
+	//  because Object can be returned stringified from Dynamic::Var::toString(),
+	//  so it must know whether to escape unicode or not.
+	bool              _escapeUnicode;
+	mutable StructPtr    _pStruct;
+	mutable OrdStructPtr _pOrdStruct;
+	mutable bool         _modified;
 };
 
 
 //
 // inlines
 //
+
+inline void Object::setEscapeUnicode(bool escape)
+{
+	_escapeUnicode = escape;
+}
+
+
+inline bool Object::getEscapeUnicode() const
+{
+	return _escapeUnicode;
+}
+
+
+inline Object::Iterator Object::begin()
+{
+	return _values.begin();
+}
+
+
+inline Object::ConstIterator Object::begin() const
+{
+	return _values.begin();
+}
+
+
+inline Object::Iterator Object::end()
+{
+	return _values.end();
+}
+
+
+inline Object::ConstIterator Object::end() const
+{
+	return _values.end();
+}
+
+
 inline bool Object::has(const std::string& key) const
 {
 	ValueMap::const_iterator it = _values.find(key);
@@ -292,8 +411,7 @@ inline bool Object::isArray(const std::string& key) const
 
 inline bool Object::isArray(ConstIterator& it) const
 {
-	const std::type_info& ti = it->second.type();
-	return it != _values.end() && (ti == typeid(Array::Ptr) || ti == typeid(Array));
+	return it != _values.end() && (it->second.type() == typeid(Array::Ptr) || it->second.type() == typeid(Array));
 }
 
 
@@ -313,8 +431,7 @@ inline bool Object::isObject(const std::string& key) const
 
 inline bool Object::isObject(ConstIterator& it) const
 {
-	const std::type_info& ti = it->second.type();
-	return it != _values.end() && (ti == typeid(Object::Ptr) || ti == typeid(Object));
+	return it != _values.end() && (it->second.type() == typeid(Object::Ptr) || it->second.type() == typeid(Object));
 }
 
 
@@ -326,20 +443,20 @@ inline std::size_t Object::size() const
 
 inline void Object::remove(const std::string& key)
 {
-	_values.erase(key);
 	if (_preserveInsOrder)
 	{
-		KeyPtrList::iterator it = _keys.begin();
-		KeyPtrList::iterator end = _keys.end();
+		KeyList::iterator it = _keys.begin();
+		KeyList::iterator end = _keys.end();
 		for (; it != end; ++it)
 		{
-			if (key == **it)
+			if (key == (*it)->first)
 			{
 				_keys.erase(it);
 				break;
 			}
 		}
 	}
+	_values.erase(key);
 	_modified = true;
 }
 
@@ -356,9 +473,9 @@ inline const Dynamic::Var& Object::getValue(ValueMap::const_iterator& it) const
 }
 
 
-inline const Dynamic::Var& Object::getValue(KeyPtrList::const_iterator& it) const
+inline const Dynamic::Var& Object::getValue(KeyList::const_iterator& it) const
 {
-	ValueMap::const_iterator itv = _values.find(**it);
+	ValueMap::const_iterator itv = _values.find((*it)->first);
 	if (itv != _values.end())
 		return itv->second;
 	else
@@ -453,7 +570,7 @@ public:
 	void convert(std::string& s) const
 	{
 		std::ostringstream oss;
-		_val->stringify(oss, 2);
+		_val->stringify(oss);
 		s = oss.str();
 	}
 
@@ -595,7 +712,7 @@ public:
 	void convert(std::string& s) const
 	{
 		std::ostringstream oss;
-		_val.stringify(oss, 2);
+		_val.stringify(oss);
 		s = oss.str();
 	}
 
