@@ -25,12 +25,14 @@
 	#endif
 #endif
 
+// disable min/max macros
+#define NOMINMAX
 
 #if !defined(POCO_NO_WINDOWS_H)
     #include <windows.h>
     #ifdef __MINGW32__
-        #include <Winsock2.h>
-        #include <Iphlpapi.h>
+        #include <winsock2.h>
+        #include <iphlpapi.h>
         #include <ws2tcpip.h>
     #endif // __MINGW32__
 #endif
